/**
 * State object
 * @typedef {Object} HttpState
 * @private
 * @property data - The response body (as JSON)
 * @property response - The HTTP response from the OpenMRS server (excluding the body)
 * @property references - An array containing all previous data objects
 * @private
 */
/**
 * Options object
 * @typedef {Object} HTTPRequestOptions
 * @property {object} query - An object of query parameters to be encoded into the URL
 * @property {object} headers - An object of all request headers
 * @property {object} body - The request body (as JSON)
 * @property {object|boolean} errors - Pass `false` to not throw on errors. Pass a map of errorCodes: error messages, ie, `{ 404: 'Resource not found' }`, or `false` to suppress errors for a specific code.
 * @property {string} [parseAs='json'] - The response format to parse (e.g., 'json', 'text', or 'stream')
 */
/**
 * Make a HTTP request to any OpenMRS endpoint
 * @example <caption>GET request with a query parameters</caption>
 * http.request("GET",
 *   "/ws/rest/v1/patient/d3f7e1a8-0114-4de6-914b-41a11fc8a1a8", {
 *    query:{
 *       limit: 1,
 *       startIndex: 20
 *    },
 * });
 * @function
 * @public
 * @param {string} method - HTTP method to use
 * @param {string} path - Path to resource
 * @param {HTTPRequestOptions}  [options={}] - An object containing query, headers, and body for the request
 * @state {HttpState}
 * @returns {Operation}
 */
export function request(method: string, path: string, options?: HTTPRequestOptions): Operation;
/**
 * Make a GET request to any OpenMRS endpoint.
 * Unlike the main `get()`, this does not append anything to the path you provide.
 * @public
 * @function
 * @example <caption>GET a resource with a query</caption>
 * http.get(
 *  "/ws/rest/v1/patient",
 *  {
 *    query: {
 *      limit: 1
 *    }
 *  }
 * )
 * @example <caption>Don't throw if OpenMRS returns a 404 error code</caption>
 * http.get(
 *  "/ws/rest/v1/patient",
 *  {
 *    errors: { 404: false }
 *  }
 * )
 * @param {string} path - path to resource
 * @param {HTTPRequestOptions} [options={}] - An object containing query params and headers for the request
 * @state {HttpState}
 * @returns {operation}
 */
export function get(path: string, options?: HTTPRequestOptions): operation;
/**
 * Make a POST request to an OpenMRS endpoint
 * @public
 * @function
 * @example <caption>Post with a JSON payload</caption>
 * http.post(
 *  "/ws/rest/v1/patient",
 *  {
 *      "person": {
 *      "gender":"M",
 *      "age":47,
 *      "birthdate":"1970-01-01T00:00:00.000+0100",
 *      "names":[
 *        {
 *          "givenName":"Jon",
 *          "familyName":"Snow"
 *        }
 *      ],
 *    }
 *    }
 * )
 * @param {string} path - path to resource
 * @param {any} data - the payload
 * @param {HTTPRequestOptions} [options={}] - An object containing query params and headers for the request
 * @state {HttpState}
 * @returns {operation}
 */
export function post(path: string, data: any, options?: HTTPRequestOptions): operation;
/**
 * State object
 */
export type HttpState = any;
/**
 * Options object
 */
export type HTTPRequestOptions = {
    /**
     * - An object of query parameters to be encoded into the URL
     */
    query: object;
    /**
     * - An object of all request headers
     */
    headers: object;
    /**
     * - The request body (as JSON)
     */
    body: object;
    /**
     * - Pass `false` to not throw on errors. Pass a map of errorCodes: error messages, ie, `{ 404: 'Resource not found' }`, or `false` to suppress errors for a specific code.
     */
    errors: object | boolean;
    /**
     * - The response format to parse (e.g., 'json', 'text', or 'stream')
     */
    parseAs?: string;
};
/**
 * Make a DELETE request to an OpenMRS endpoint
 * @alias delete
 * @public
 * @function
 * @example <caption>Delete a resource</caption>
 * http.delete(
 *  "/ws/rest/v1/patient/abc/"
 * )
 * @param {string} path - path to resource
 * @param {HTTPRequestOptions} [options={}] - An object containing query params and headers for the request
 * @state {HttpState}
 * @returns {operation}
 */
declare function _delete(path: string, options?: HTTPRequestOptions): operation;
export { _delete as delete, _delete as delete };
